#!/usr/bin/perl
#
# $Id: functions.pl,v 1.13 2000/10/03 13:35:53 cvs Exp $
#
# This file contains functions used by the mailscan system

use MIME::Parser;

require '/etc/mailscan/mailscan.conf';
$msglog = "";

# &inlist($string, @list)
#  Returns 1 if the string $string appears in @list
sub inlist {
  my $findstr = shift;
  my @list = @_;
  my $res = 0;
  my $lookat;
  foreach $lookat (@list) {
    if ($lookat eq $findstr) {
      $res = 1;
    }
  }
  return $res;
}

# &rinlist($re, @list)
#  Returns 1 if the regular expression $re matches a string in @list
sub rinlist {
  my $findstr = shift;
  my @list = @_;
  my $res = 0;
  my $lookat;
  foreach $lookat (@list) {
    if ($lookat =~ /$findstr/i) {
      $res = 1;
    }
  }
  return $res;
}

# &inrlist($string, @list)
#  Returns 1 if the string $string matches a regular expression in @list
sub inrlist {
  my $findstr = shift;
  my @list = @_;
  my $res = 0;
  my $lookat;
  foreach $lookat (@list) {
    if ($findstr =~ /$lookat/i) {
      $res = 1;
    }
  }
  return $res;
}

# &replacer($string, %hash)
#  Parses $string, replacing instances of %key% with the value associated with
#  key in %hash if found
sub replacer {
  my ($inp) = shift;
  my ($rlist) = shift;
  foreach $rstr (keys %{$rlist}) {
    my $rwith = $rlist->{$rstr};
    $rstr =~ s/\@/\\\@/;
    $rstr = '%' . $rstr . '%';
    my $exp = "\$inp =~ s/%" . $rstr . "%/" . $rwith . "/g;";
    #eval $exp;
    $inp =~ s/$rstr/$rwith/g;
  }
  return $inp;
}

# &Log($string)
#  Writes $string to the log file
sub Log {
  open(L,">>".$CONF->{'params'}->{'logfile'});
  print L scalar localtime, " : ", $HDR{'DEST_ID'}, " : ", @_, "\n";
  close(L);
}

# &deliver()
#  Pass the message on to the real delivery agent
sub deliver {
  my $rec = $HDR{'RCPT_TO'};
  $pid = open(KID,"|-");
  if ($pid) {
    $entity->print(\*KID);
    close KID;
  } else {
   exec "/usr/lib/sendmail", "-C/etc/mail/sendmail.cf", "-f", $HDR{"MAIL_FROM"}, $rec;
  }
  $msglog = $msglog . " Stat=Delivered,";
}

# &sendmsg($msgname)
#  Send out message $msgname from the config
sub sendmsg {
  my ($msgname) = shift;
  if ($CONF->{'msg'}->{$msgname}) {
    my $sender = replacer($CONF->{'msg'}->{$msgname}->{'sender'}, \%HDR);
    my $recips = replacer($CONF->{'msg'}->{$msgname}->{'recips'}, \%HDR);
    my $msgtxt = replacer($CONF->{'msg'}->{$msgname}->{'msg'}, \%HDR);
    while ($msgtxt =~ /%(\S+)%/) {
      my $f = $1;
      my $r = $entity->head->get($f);
      $f = '%' . $f . '%';
      $r =~ s/\n$//; $r =~ s/\r$//;
      $msgtxt =~ s/$f/$r/;
      #my $e = '$msgtxt =~ s/%'.$f.'%/'.$r.'/';
      #eval "$e";
    }
    $pid = open(KID,"|-");
    if ($pid) {
      print KID $msgtxt;
      close KID;
    } else {
     exec "/usr/lib/sendmail", "-C/etc/mail/sendmail.cf", "-oi", "-f", $sender,
$recips;
    }
    $msglog = $msglog . " SendMsg=$msgname,";
  }
}

# &addatt($attname)
#  Attaches $attname as a text/plain attachment to the message
sub addatt {
  my ($attname) = shift;
  if ($CONF->{'att'}->{$attname}) {
    my $msgtxt = replacer($CONF->{'att'}->{$attname}, \%HDR);
    while ($msgtxt =~ /%(.*)%/) {
      my $f = $1;
      my $r = $entity->head->get($f);
      $r =~ s/\@/\\\@/g;
      $r =~ s/\n$//; $r =~ s/\r$//;
      my $e = '$msgtxt =~ s/%'.$f.'%/'.$r.'/';
      eval "$e";
    }
    $entity->attach(Data => $msgtxt);
    $msglog = $msglog . " AddPart=$attname,";
  }
}

# &mv2q($qname)
#  Moves the message into queue $qname, executing the "onmove" block if needed,
#  this routine also stops processing the message
sub mv2q {
  my ($qname) = shift;
  if ($CONF->{'queue'}->{$qname}) {
    open(O,">".$CONF->{'params'}->{'basedir'}."/".$qname."/".$HDR{'DEST_ID'}.".msg");
    $entity->print(\*O);
    close(O);
    &delfile($CONF->{'params'}->{'basedir'}."/".$HDR{'DEST_ID'}.".msg");
    open(I,"<".$CONF->{'params'}->{'basedir'}."/".$HDR{'DEST_ID'}.".pars");
    open(O,">".$CONF->{'params'}->{'basedir'}."/".$qname."/".$HDR{'DEST_ID'}.".pars");
    while (<I>) {
      print O $_;
    }
    close(O);
    close(I);
    &delfile($CONF->{'params'}->{'basedir'}."/".$HDR{'DEST_ID'}.".pars");
    if ($CONF->{'queue'}->{$qname}->{'onmove'}) {
      eval $CONF->{'queue'}->{$qname}->{'onmove'};
    }
    $msglog = $msglog . " Move2=$qname,";
    &endnow;
  }
}

# zpad($length, $number)
#  Left-pads $number with "0" to $length
sub zpad {
  my ($nd) = shift;
  my ($s) = shift;
  while (length($s) < $nd) {
    $s = "0" . $s;
  }
  return $s;
}

# &idnum()
#  Generate a message ID string
sub idnum {
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time);
  return zpad(4,(1900 + $year)).zpad(2,(1 + $mon)).zpad(2,$mday).zpad(2,$hour).zpad(2,$min).".".zpad(6,$$);
}

# endnow()
#  Clean up and exit
sub endnow {
  if ($CONF->{'params'}->{'endlog'}) {
    $msglog = $msglog . " Done";
    &Log("MSGLOG: ",$msglog);
  }
  my $p = $parser->output_dir;
  if (($p =~ /msg-/) && (-d $p)) {
    my $rlin;
    my $pid = open(KID, "-|");
    if ($pid) {
      while ($rlin = <KID>) { };
      close(KID);
    } else {
      exec "/bin/rm", "-vrf", "$p";
    }
  }
  exit(0);
}

# delfile($filename)
#  suid-root "safe" file delete
sub delfile {
  my $p = shift;
  my $rlin;
  my $pid = open(KID, "-|");
  if ($pid) {
    while ($rlin = <KID>) { };
    close(KID);
  } else {
    exec "/bin/rm", "-vf", "$p";
  }
}

# &getheader($hdrname)
#  Return header $hdrname from the messages, combines duplicate headers into
#  one string. $hdrname is not case-sensitive
sub getheader {
  my $hdrname = shift;
  my $ret = "";
  my $headers = $entity->head;
  if ($headers->count($hdrname)) {
    if ($headers->count($hdrname) == 1) {
      $ret = $headers->get($hdrname);
    } else {
      my @hdrarr = $headers->get($hdrname);
      my $hlin;
      foreach $hlin (@hdrarr) {
        $ret .= $hlin;
      }
    }
  }
  return $ret;
}

# "require" files must end with 1;
1;

